@@ -82,7 +82,7 @@ struct _AnjutaShellIface {
 	AnjutaProfileManager* (*get_profile_manager) (AnjutaShell *shell,
 												  GError **err);
 	
-	void (*add_widget_full)        (AnjutaShell  *shell,
+	void (*add_widget_full)   (AnjutaShell  *shell,
 							   GtkWidget    *widget,
 							   const char   *name,
 							   const char   *title,
@@ -90,6 +90,14 @@ struct _AnjutaShellIface {
 							   AnjutaShellPlacement placement,
 							   gboolean locked,
 							   GError      **error);
+	void (*add_widget_custom)  (AnjutaShell  *shell,
+	                            GtkWidget    *widget,
+	                            const char   *name,
+	                            const char   *title,
+	                            const char   *stock_id,
+	                            GtkWidget    *label,
+	                            AnjutaShellPlacement placement,
+	                            GError      **error);
 	
 	void (*remove_widget)     (AnjutaShell  *shell,
 							   GtkWidget    *widget,
@@ -97,6 +105,21 @@ struct _AnjutaShellIface {
 	void (*present_widget)    (AnjutaShell  *shell,
 							   GtkWidget    *widget,
 							   GError      **error);
+	
+	void (*iconify_dockable_widget) (AnjutaShell *shell, 
+                               GtkWidget    *widget,
+                               GError      **error);
+	void (*hide_dockable_widget) (AnjutaShell *shell, 
+                               GtkWidget    *widget,
+                               GError      **error);
+	void (*show_dockable_widget) (AnjutaShell *shell, 
+                               GtkWidget    *widget,
+                               GError      **error);
+	void (*maximize_widget)   (AnjutaShell  *shell,
+	                           const char   *widget_name,
+                               GError      **error);
+	void (*unmaximize)        (AnjutaShell  *shell,
+                               GError      **error);
 	void (*add_value)         (AnjutaShell  *shell,
 							   const char   *name,
 							   const GValue *value,
@@ -108,6 +131,8 @@ struct _AnjutaShellIface {
 	void (*remove_value)      (AnjutaShell  *shell,
 							   const char   *name,
 							   GError       **error);
+	void (*saving_push)       (AnjutaShell* shell);
+	void (*saving_pop)        (AnjutaShell* shell);
 	GObject* (*get_object)    (AnjutaShell  *shell,
 							   const char   *iface_name,
 							   GError       **error);
@@ -140,6 +165,15 @@ void   anjuta_shell_add_widget      (AnjutaShell     *shell,
 									 AnjutaShellPlacement placement,
 									 GError         **error);
 
+void   anjuta_shell_add_widget_custom (AnjutaShell   *shell,
+                                       GtkWidget     *widget,
+                                       const char    *name,
+                                       const char    *title,
+                                       const char    *stock_id,
+                                       GtkWidget     *label,
+                                       AnjutaShellPlacement placement,
+                                       GError        **error);
+
 void   anjuta_shell_add_widget_full      (AnjutaShell     *shell,
 									 GtkWidget       *widget,
 									 const char      *name,
@@ -155,6 +189,20 @@ void   anjuta_shell_remove_widget   (AnjutaShell     *shell,
 void   anjuta_shell_present_widget  (AnjutaShell     *shell,
 									 GtkWidget       *widget,
 									 GError         **error);
+void anjuta_shell_iconify_dockable_widget (AnjutaShell *shell, 
+                                     GtkWidget   *widget,
+                                     GError      **error);
+void anjuta_shell_hide_dockable_widget (AnjutaShell *shell, 
+                                     GtkWidget   *widget,
+                                     GError      **error);
+void anjuta_shell_show_dockable_widget (AnjutaShell *shell, 
+                                     GtkWidget   *widget,
+                                     GError      **error);
+void anjuta_shell_maximize_widget   (AnjutaShell *shell,
+                                     const char  *widget_name,
+                                     GError      **error);
+void anjuta_shell_unmaximize        (AnjutaShell *shell,
+                                     GError      **error);
 void   anjuta_shell_add_value       (AnjutaShell     *shell,
 									 const char      *name,
 									 const GValue    *value,
@@ -196,6 +244,9 @@ void anjuta_shell_save_prompt       (AnjutaShell *shell,
 									 AnjutaSavePrompt *prompt,
 									 GError **error);
 void anjuta_shell_notify_exit       (AnjutaShell *shell, GError **error);
+void anjuta_shell_saving_push	    (AnjutaShell* shell);
+void anjuta_shell_saving_pop	    (AnjutaShell* shell);
+
 
 /**
  * anjuta_shell_get_interface:
