@@ -27,12 +27,9 @@
 #include <ctype.h>
 #include <sys/wait.h>
 
-#include <glade/glade-xml.h>
 #include <gtk/gtk.h>
 
-#include <gdl/gdl-dock.h>
-#include <gdl/gdl-dock-bar.h>
-#include <gdl/gdl-switcher.h>
+#include <gdl/gdl.h>
 
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-ui.h>
@@ -45,8 +42,8 @@
 #include "anjuta-actions.h"
 #include "about.h"
 
-#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta.ui"
-#define GLADE_FILE PACKAGE_DATA_DIR"/glade/anjuta.glade"
+#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta.xml"
+#define GLADE_FILE PACKAGE_DATA_DIR"/glade/preferences.ui"
 #define ICON_FILE "anjuta-preferences-general-48.png"
 
 static void anjuta_app_layout_load (AnjutaApp *app,
@@ -60,13 +57,205 @@ static gpointer parent_class = NULL;
 static GtkToolbarStyle style = -1;
 
 static void
-on_toolbar_style_changed (GConfClient* client, guint id, GConfEntry* entry,
-						  gpointer user_data)
+menu_item_select_cb (GtkMenuItem *proxy,
+                     AnjutaApp *app)
+{
+	GtkAction *action;
+	char *message;
+
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+	g_return_if_fail (action != NULL);
+
+	g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+	if (message)
+	{
+		anjuta_status_push (app->status, "%s", message);
+		g_free (message);
+	}
+}
+
+static void
+menu_item_deselect_cb (GtkMenuItem *proxy,
+                       AnjutaApp *app)
+{
+	anjuta_status_pop (app->status);
+}
+
+
+static void
+connect_proxy_cb (GtkUIManager *manager,
+                  GtkAction *action,
+                  GtkWidget *proxy,
+                  AnjutaApp *app)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		g_signal_connect (proxy, "select",
+				  G_CALLBACK (menu_item_select_cb), app);
+		g_signal_connect (proxy, "deselect",
+				  G_CALLBACK (menu_item_deselect_cb), app);
+	}
+}
+
+static void
+disconnect_proxy_cb (GtkUIManager *manager,
+                     GtkAction *action,
+                     GtkWidget *proxy,
+	                 AnjutaApp *app)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		g_signal_handlers_disconnect_by_func
+			(proxy, G_CALLBACK (menu_item_select_cb), app);
+		g_signal_handlers_disconnect_by_func
+			(proxy, G_CALLBACK (menu_item_deselect_cb), app);
+	}
+}
+
+static void
+anjuta_app_iconify_dockable_widget (AnjutaShell *shell, GtkWidget *widget,
+                                    GError **error)
+{
+	AnjutaApp *app = NULL;
+	GtkWidget *dock_item = NULL;
+
+	/* Argumments assertions */
+	g_return_if_fail (ANJUTA_IS_APP (shell));
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	app = ANJUTA_APP (shell);
+	g_return_if_fail (app->widgets != NULL);
+	
+	dock_item = g_object_get_data (G_OBJECT (widget), "dockitem");
+	g_return_if_fail (dock_item != NULL);
+
+	/* Iconify the dockable item */
+	gdl_dock_item_iconify_item (GDL_DOCK_ITEM (dock_item));
+}
+
+static void
+anjuta_app_hide_dockable_widget (AnjutaShell *shell, GtkWidget *widget,
+                                 GError **error)
+{
+	AnjutaApp *app = NULL;
+	GtkWidget *dock_item = NULL;
+
+	/* Argumments assertions */
+	g_return_if_fail (ANJUTA_IS_APP (shell));
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	app = ANJUTA_APP (shell);
+	g_return_if_fail (app->widgets != NULL);
+	
+	dock_item = g_object_get_data (G_OBJECT (widget), "dockitem");
+	g_return_if_fail (dock_item != NULL);
+
+	/* Hide the dockable item */
+	gdl_dock_item_hide_item (GDL_DOCK_ITEM (dock_item));
+}
+
+static void
+anjuta_app_show_dockable_widget (AnjutaShell *shell, GtkWidget* widget,
+                                 GError **error)
+{
+	AnjutaApp *app = NULL;
+	GtkWidget *dock_item = NULL;
+
+	/* Argumments assertions */
+	g_return_if_fail (ANJUTA_IS_APP (shell));
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	app = ANJUTA_APP (shell);
+	g_return_if_fail (app->widgets != NULL);
+	
+	dock_item = g_object_get_data (G_OBJECT (widget), "dockitem");
+	g_return_if_fail (dock_item != NULL);
+
+	/* Show the dockable item */	
+	gdl_dock_item_show_item(GDL_DOCK_ITEM (dock_item));
+}
+
+static void
+anjuta_app_maximize_widget (AnjutaShell *shell,
+                            const char  *widget_name,
+                            GError **error)
+{
+	AnjutaApp *app = NULL;
+	GtkWidget *dock_item = NULL;
+	gpointer  value, key;
+	GtkWidget *widget = NULL;
+	GHashTableIter iter;
+
+	/* AnjutaApp assertions */
+	g_return_if_fail (ANJUTA_IS_APP (shell));
+	app = ANJUTA_APP (shell);
+
+	/* If app->maximized is TRUE then another widget is already maximized.
+	   Restoring the UI for a new maximization. */
+	if(app->maximized)
+		gdl_dock_layout_load_layout (app->layout_manager, "back-up");
+
+	/* Back-up the layout so it can be restored */
+	gdl_dock_layout_save_layout(app->layout_manager, "back-up");
+
+	/* Mark the app as maximized (the other widgets except center are hidden) */
+	app->maximized = TRUE;
+	
+	/* Hide all DockItem's except the ones positioned in the center */
+	g_hash_table_iter_init (&iter, app->widgets);
+	while (g_hash_table_iter_next (&iter, &key, &value))
+	{
+		if (value == NULL)
+			continue;
+
+		/* If it's the widget requesting maximization then continue */
+		if(!g_strcmp0((gchar*)key, widget_name))
+			continue;
+			
+		/* Widget assertions */
+		widget = GTK_WIDGET (value);
+		if(!GTK_IS_WIDGET (widget))
+			continue;
+
+		/* DockItem assertions */
+		dock_item = g_object_get_data (G_OBJECT (widget), "dockitem");
+		if(dock_item == NULL || !GDL_IS_DOCK_ITEM (dock_item))
+			continue;
+		
+		/* Hide the item */
+		gdl_dock_item_hide_item (GDL_DOCK_ITEM (dock_item));
+	}
+}
+
+static void
+anjuta_app_unmaximize (AnjutaShell *shell,
+                       GError **error)
+{
+	AnjutaApp *app = NULL;
+
+	/* AnjutaApp assertions */
+	g_return_if_fail (ANJUTA_IS_APP (shell));
+	app = ANJUTA_APP (shell);
+
+	/* If not maximized then the operation doesn't make sence. */
+	g_return_if_fail (app->maximized);
+	
+	/* Load the backed-up layout */
+	gdl_dock_layout_load_layout (app->layout_manager, "back-up");
+	gdl_dock_layout_delete_layout (app->layout_manager, "back-up");
+	
+	/* Un-mark maximized */
+	app->maximized = FALSE;
+}
+
+static void
+on_toolbar_style_changed (AnjutaPreferences* prefs,
+                          const gchar* key,
+                          const gchar* tb_style,
+                          gpointer user_data)
 {
 	AnjutaApp* app = ANJUTA_APP (user_data);
-	
-	char* tb_style = anjuta_preferences_get (app->preferences, "anjuta.toolbar.style");
-	
+		
 	if (tb_style)
 	{	
 		if (strcasecmp (tb_style, "Default") == 0)
@@ -81,8 +270,6 @@ on_toolbar_style_changed (GConfClient* client, guint id, GConfEntry* entry,
 			style = GTK_TOOLBAR_TEXT;
 		
 		DEBUG_PRINT ("Toolbar style: %s", tb_style);
-		
-		g_free(tb_style);
 	}
 	
 	if (style != -1)
@@ -96,14 +283,14 @@ on_toolbar_style_changed (GConfClient* client, guint id, GConfEntry* entry,
 }
 
 static void
-on_gdl_style_changed (GConfClient* client, guint id, GConfEntry* entry,
-					  gpointer user_data)
+on_gdl_style_changed (AnjutaPreferences* prefs,
+                      const gchar* key,
+                      const gchar* pr_style,
+                      gpointer user_data)
 {
 	AnjutaApp* app = ANJUTA_APP (user_data);
 	GdlSwitcherStyle style = GDL_SWITCHER_STYLE_BOTH;
 	
-	gchar* pr_style = anjuta_preferences_get (app->preferences, "anjuta.gdl.style");
-
 	if (pr_style)
 	{
 		if (strcasecmp (pr_style, "Text") == 0)
@@ -118,8 +305,6 @@ on_gdl_style_changed (GConfClient* client, guint id, GConfEntry* entry,
 			style = GDL_SWITCHER_STYLE_TABS;
 		
 		DEBUG_PRINT ("Switcher style: %s", pr_style);
-		
-		g_free (pr_style);
 	}
 	g_object_set (G_OBJECT(app->layout_manager->master), "switcher-style",
 				  style, NULL);
@@ -202,7 +387,7 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 		return;
 	
 	/* Save geometry */
-	state = gdk_window_get_state (GTK_WIDGET (app)->window);
+	state = gdk_window_get_state (gtk_widget_get_window (GTK_WIDGET (app)));
 	if (state & GDK_WINDOW_STATE_MAXIMIZED) {
 		anjuta_session_set_int (session, "Anjuta", "Maximized", 1);
 	}
@@ -350,6 +535,7 @@ anjuta_app_instance_init (AnjutaApp *app)
 	GtkWidget *dockbar;
 	GtkAction* action;
 	GList *plugins_dirs = NULL;
+	gchar* style;
 	GdkGeometry size_hints = {
     	100, 100, 0, 0, 100, 100, 1, 1, 0.0, 0.0, GDK_GRAVITY_NORTH_WEST
   	};
@@ -369,6 +555,7 @@ anjuta_app_instance_init (AnjutaApp *app)
 	
 	app->values = NULL;
 	app->widgets = NULL;
+	app->maximized = FALSE;
 	
 	/* Status bar */
 	app->status = ANJUTA_STATUS (anjuta_status_new ());
@@ -399,6 +586,15 @@ anjuta_app_instance_init (AnjutaApp *app)
 	/* UI engine */
 	app->ui = anjuta_ui_new ();
 	g_object_add_weak_pointer (G_OBJECT (app->ui), (gpointer)&app->ui);
+	/* show tooltips in the statusbar */
+	g_signal_connect (app->ui,
+			  "connect_proxy",
+			  G_CALLBACK (connect_proxy_cb),
+			  app);
+	g_signal_connect (app->ui,
+			  "disconnect_proxy",
+			  G_CALLBACK (disconnect_proxy_cb),
+			  app);
 	
 	/* Plugin Manager */
 	plugins_dirs = g_list_prepend (plugins_dirs, PACKAGE_PLUGIN_DIR);
@@ -413,10 +609,13 @@ anjuta_app_instance_init (AnjutaApp *app)
 	g_object_add_weak_pointer (G_OBJECT (app->preferences),
 							   (gpointer)&app->preferences);
 	
-	anjuta_preferences_notify_add (app->preferences, "anjuta.gdl.style",
-								   on_gdl_style_changed, app, NULL);
+	anjuta_preferences_notify_add_string (app->preferences, "anjuta.gdl.style",
+	                                      on_gdl_style_changed, app, NULL);
+	style = anjuta_preferences_get (app->preferences, "anjuta.gdl.style");
 	
-	on_gdl_style_changed (NULL, 0, NULL, app);
+	on_gdl_style_changed (app->preferences, NULL, 
+	                      style, app);
+	g_free (style);
 	
 	/* Register actions */
 	anjuta_ui_add_action_group_entries (app->ui, "ActionGroupFile", _("File"),
@@ -457,16 +656,20 @@ anjuta_app_instance_init (AnjutaApp *app)
 	/* create toolbar */	
 	app->toolbar = gtk_ui_manager_get_widget (GTK_UI_MANAGER (app->ui),
 										 "/ToolbarMain");
+    if (!anjuta_preferences_get_bool (app->preferences, "anjuta.toolbar.visible"))
+		gtk_widget_hide (app->toolbar);
 	gtk_box_pack_start (GTK_BOX (main_box), app->toolbar, FALSE, FALSE, 0);
 	action = gtk_ui_manager_get_action (GTK_UI_MANAGER (app->ui),
 										"/MenuMain/MenuView/Toolbar");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION(action),
-								  anjuta_preferences_get_int_with_default (app->preferences,
+								  anjuta_preferences_get_bool_with_default (app->preferences,
 																		   "anjuta.toolbar.visible",
 																		   TRUE));
-	anjuta_preferences_notify_add (app->preferences, "anjuta.toolbar.style",
+	anjuta_preferences_notify_add_string (app->preferences, "anjuta.toolbar.style",
 								   on_toolbar_style_changed, app, NULL);
-	on_toolbar_style_changed (NULL, 0, NULL, app);
+	style = anjuta_preferences_get (app->preferences, "anjuta.toolbar.style");
+	on_toolbar_style_changed (app->preferences, NULL, style, app);
+	g_free (style);
 
 	/* Create widgets menu */
 	view_menu = 
@@ -498,6 +701,42 @@ anjuta_app_instance_init (AnjutaApp *app)
 	
 	/* Loading accels */
 	anjuta_ui_load_accels (NULL);
+
+	app->save_count = 0;
+}
+
+/*
+ * GtkWindow catches keybindings for the menu items _before_ passing them to
+ * the focused widget. This is unfortunate and means that pressing ctrl+V
+ * in an entry on a panel ends up pasting text in the TextView.
+ * Here we override GtkWindow's handler to do the same things that it
+ * does, but in the opposite order and then we chain up to the grand
+ * parent handler, skipping gtk_window_key_press_event.
+ */
+static gboolean
+anjuta_app_key_press_event (GtkWidget   *widget,
+                            GdkEventKey *event)
+{
+	static gpointer grand_parent_class = NULL;
+	GtkWindow *window = GTK_WINDOW (widget);
+	gboolean handled = FALSE;
+
+	if (grand_parent_class == NULL)
+		grand_parent_class = g_type_class_peek_parent (parent_class);
+
+	/* handle focus widget key events */
+	if (!handled)
+		handled = gtk_window_propagate_key_event (window, event);
+
+	/* handle mnemonics and accelerators */
+	if (!handled)
+		handled = gtk_window_activate_key (window, event);
+
+	/* Chain up, invokes binding set */
+	if (!handled)
+		handled = GTK_WIDGET_CLASS (grand_parent_class)->key_press_event (widget, event);
+
+	return handled;
 }
 
 static void
@@ -509,8 +748,11 @@ anjuta_app_class_init (AnjutaAppClass *class)
 	parent_class = g_type_class_peek_parent (class);
 	object_class = (GObjectClass*) class;
 	widget_class = (GtkWidgetClass*) class;
+	
 	object_class->finalize = anjuta_app_finalize;
 	object_class->dispose = anjuta_app_dispose;
+
+	widget_class->key_press_event = anjuta_app_key_press_event;
 }
 
 GtkWidget *
@@ -534,7 +776,7 @@ anjuta_app_get_geometry (AnjutaApp *app)
 	
 	geometry = NULL;
 	width = height = posx = posy = 0;
-	if (GTK_WIDGET(app)->window)
+	if (gtk_widget_get_window (GTK_WIDGET (app)))
 	{
 		gtk_window_get_size (GTK_WINDOW (app), &width, &height);
 		gtk_window_get_position (GTK_WINDOW(app), &posx, &posy);
@@ -596,6 +838,11 @@ anjuta_app_layout_save (AnjutaApp *app, const gchar *filename,
 	g_return_if_fail (ANJUTA_IS_APP (app));
 	g_return_if_fail (filename != NULL);
 
+	/* If maximized, the layout should be loaded from the back-up first */
+	if(app->maximized)
+		gdl_dock_layout_load_layout (app->layout_manager, "back-up");
+
+	/* Continue with the saving */
 	gdl_dock_layout_save_layout (app->layout_manager, name);
 	if (!gdl_dock_layout_save_to_file (app->layout_manager, filename))
 		g_warning ("Saving dock layout to '%s' failed!", filename);
@@ -637,14 +884,22 @@ anjuta_app_layout_reset (AnjutaApp *app)
 void 
 anjuta_app_install_preferences (AnjutaApp *app)
 {
-	GladeXML *gxml;	
+	GtkBuilder* builder = gtk_builder_new ();
+	GError* error = NULL;
 	GtkWidget *notebook, *shortcuts, *plugins, *remember_plugins;
 	
 	/* Create preferences page */
-	gxml = glade_xml_new (GLADE_FILE, "anjuta_preferences_window", NULL);
-	anjuta_preferences_add_page (app->preferences, gxml,
+	gtk_builder_add_from_file (builder, GLADE_FILE, &error);
+	if (error)
+	{
+		g_warning("Could not load general preferences: %s",
+			  error->message);
+		g_error_free (error);
+		return;
+	}
+	anjuta_preferences_add_from_builder (app->preferences, builder, 
 								 "General", _("General"), ICON_FILE);
-	notebook = 	glade_xml_get_widget (gxml, "General");
+	notebook = 	GTK_WIDGET (gtk_builder_get_object (builder, "General"));
 	shortcuts = anjuta_ui_get_accel_editor (ANJUTA_UI (app->ui));
 	plugins = anjuta_plugin_manager_get_plugins_page (app->plugin_manager);
 	remember_plugins = anjuta_plugin_manager_get_remembered_plugins_page (app->plugin_manager);
@@ -660,7 +915,7 @@ anjuta_app_install_preferences (AnjutaApp *app)
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), shortcuts,
 							  gtk_label_new (_("Shortcuts")));
 	
-	g_object_unref (gxml);
+	g_object_unref (builder);
 }
 
 /* AnjutaShell Implementation */
@@ -768,6 +1023,20 @@ anjuta_app_remove_value (AnjutaShell *shell, const char *name, GError **error)
 	}
 }
 
+static void
+anjuta_app_saving_push (AnjutaShell* shell)
+{
+	AnjutaApp* app = ANJUTA_APP (shell);
+	app->save_count++;
+}
+
+static void
+anjuta_app_saving_pop (AnjutaShell* shell)
+{
+	AnjutaApp* app = ANJUTA_APP (shell);
+	app->save_count--;
+}
+
 static gboolean
 remove_from_widgets_hash (gpointer name, gpointer hash_widget, gpointer widget)
 {
@@ -831,32 +1100,16 @@ on_widget_removed_from_hash (gpointer widget)
 	g_object_unref (G_OBJECT (widget));
 }
 
-static void 
-anjuta_app_add_widget_full (AnjutaShell *shell, 
-					   GtkWidget *widget,
-					   const char *name,
-					   const char *title,
-					   const char *stock_id,
-					   AnjutaShellPlacement placement,
-					   gboolean locked,
-					   GError **error)
+static void
+anjuta_app_setup_widget (AnjutaApp* app,
+                         const gchar* name,
+                         GtkWidget *widget,
+                         GtkWidget* item,
+                         const gchar* title,
+                         gboolean locked)
 {
-	AnjutaApp *app;
-	GtkWidget *item;
 	GtkCheckMenuItem* menuitem;
 
-	g_return_if_fail (ANJUTA_IS_APP (shell));
-	g_return_if_fail (GTK_IS_WIDGET (widget));
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (title != NULL);
-
-	app = ANJUTA_APP (shell);
-	
-	/*
-	anjuta_shell_add (shell, name, G_TYPE_FROM_INSTANCE (widget),
-					  widget, NULL);
-	*/
-
 	/* Add the widget to hash */
 	if (app->widgets == NULL)
 	{
@@ -867,32 +1120,11 @@ anjuta_app_add_widget_full (AnjutaShell *shell,
 	g_hash_table_insert (app->widgets, g_strdup (name), widget);
 	g_object_ref (widget);
 	
-	/* Add the widget to dock */
-	if (stock_id == NULL)
-		item = gdl_dock_item_new (name, title, GDL_DOCK_ITEM_BEH_NORMAL);
-	else
-		item = gdl_dock_item_new_with_stock (name, title, stock_id,
-											 GDL_DOCK_ITEM_BEH_NORMAL);
-	if (locked)
-	{
-		guint flags = 0;
-		flags |= GDL_DOCK_ITEM_BEH_NEVER_FLOATING;
-		flags |= GDL_DOCK_ITEM_BEH_CANT_CLOSE;
-		flags |= GDL_DOCK_ITEM_BEH_CANT_ICONIFY;
-		flags |= GDL_DOCK_ITEM_BEH_NO_GRIP;
-		g_object_set(G_OBJECT(item), "behavior", flags, NULL);
-	}
-	
-	gtk_container_add (GTK_CONTAINER (item), widget);
-    gdl_dock_add_item (GDL_DOCK (app->dock),
-                       GDL_DOCK_ITEM (item), placement);
-	gtk_widget_show_all (item);
-	
 	/* Add toggle button for the widget */
 	menuitem = GTK_CHECK_MENU_ITEM (gtk_check_menu_item_new_with_label (title));
 	gtk_widget_show (GTK_WIDGET (menuitem));
 	gtk_check_menu_item_set_active (menuitem, TRUE);
-	gtk_menu_append (GTK_MENU (app->view_menu), GTK_WIDGET (menuitem));
+	gtk_menu_shell_append (GTK_MENU_SHELL (app->view_menu), GTK_WIDGET (menuitem));
 
 	if (locked)
 		g_object_set( G_OBJECT(menuitem), "visible", FALSE, NULL);
@@ -914,6 +1146,95 @@ anjuta_app_add_widget_full (AnjutaShell *shell,
 					  G_CALLBACK (on_widget_remove), app);
 	g_signal_connect_after (G_OBJECT (widget), "destroy",
 					  G_CALLBACK (on_widget_destroy), app);
+
+	gtk_widget_show_all (item);
+}
+                         
+
+static void 
+anjuta_app_add_widget_full (AnjutaShell *shell, 
+					   GtkWidget *widget,
+					   const char *name,
+					   const char *title,
+					   const char *stock_id,
+					   AnjutaShellPlacement placement,
+					   gboolean locked,
+					   GError **error)
+{
+	AnjutaApp *app;
+	GtkWidget *item;
+
+	g_return_if_fail (ANJUTA_IS_APP (shell));
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (title != NULL);
+
+	app = ANJUTA_APP (shell);
+	
+	/* Add the widget to dock */
+	if (stock_id == NULL)
+		item = gdl_dock_item_new (name, title, GDL_DOCK_ITEM_BEH_NORMAL);
+	else
+		item = gdl_dock_item_new_with_stock (name, title, stock_id,
+											 GDL_DOCK_ITEM_BEH_NORMAL);
+	if (locked)
+	{
+		guint flags = 0;
+		flags |= GDL_DOCK_ITEM_BEH_NEVER_FLOATING;
+		flags |= GDL_DOCK_ITEM_BEH_CANT_CLOSE;
+		flags |= GDL_DOCK_ITEM_BEH_CANT_ICONIFY;
+		flags |= GDL_DOCK_ITEM_BEH_NO_GRIP;
+		g_object_set(G_OBJECT(item), "behavior", flags, NULL);
+	}
+	
+	gtk_container_add (GTK_CONTAINER (item), widget);
+    gdl_dock_add_item (GDL_DOCK (app->dock),
+                       GDL_DOCK_ITEM (item), placement);
+	
+	if (locked)
+		gdl_dock_item_set_default_position(GDL_DOCK_ITEM(item), GDL_DOCK_OBJECT(app->dock));
+
+	anjuta_app_setup_widget (app, name, widget, item, title, locked);
+}
+
+static void
+anjuta_app_add_widget_custom (AnjutaShell *shell, 
+                              GtkWidget *widget,
+                              const char *name,
+                              const char *title,
+                              const char   *stock_id,
+                              GtkWidget *label,
+                              AnjutaShellPlacement placement,
+                              GError **error)
+{
+	AnjutaApp *app;
+	GtkWidget *item;
+	GtkWidget *grip;
+	
+	g_return_if_fail (ANJUTA_IS_APP (shell));
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (title != NULL);
+
+	app = ANJUTA_APP (shell);
+
+	/* Add the widget to dock */
+	/* Add the widget to dock */
+	if (stock_id == NULL)
+		item = gdl_dock_item_new (name, title, GDL_DOCK_ITEM_BEH_NORMAL);
+	else
+		item = gdl_dock_item_new_with_stock (name, title, stock_id,
+											 GDL_DOCK_ITEM_BEH_NORMAL);
+
+	gtk_container_add (GTK_CONTAINER (item), widget);
+    gdl_dock_add_item (GDL_DOCK (app->dock),
+                       GDL_DOCK_ITEM (item), placement);
+	
+	grip = gdl_dock_item_get_grip (GDL_DOCK_ITEM (item));
+
+	gdl_dock_item_grip_set_label (GDL_DOCK_ITEM_GRIP (grip), label);
+
+	anjuta_app_setup_widget (app, name, widget, item, title, FALSE);
 }
 
 static void 
@@ -1026,8 +1347,14 @@ static void
 anjuta_shell_iface_init (AnjutaShellIface *iface)
 {
 	iface->add_widget_full = anjuta_app_add_widget_full;
+	iface->add_widget_custom = anjuta_app_add_widget_custom;
 	iface->remove_widget = anjuta_app_remove_widget;
 	iface->present_widget = anjuta_app_present_widget;
+	iface->iconify_dockable_widget = anjuta_app_iconify_dockable_widget;
+	iface->hide_dockable_widget = anjuta_app_hide_dockable_widget;
+	iface->show_dockable_widget = anjuta_app_show_dockable_widget;
+	iface->maximize_widget = anjuta_app_maximize_widget;
+	iface->unmaximize = anjuta_app_unmaximize;
 	iface->add_value = anjuta_app_add_value;
 	iface->get_value = anjuta_app_get_value;
 	iface->remove_value = anjuta_app_remove_value;
@@ -1037,6 +1364,8 @@ anjuta_shell_iface_init (AnjutaShellIface *iface)
 	iface->get_preferences = anjuta_app_get_preferences;
 	iface->get_plugin_manager = anjuta_app_get_plugin_manager;
 	iface->get_profile_manager = anjuta_app_get_profile_manager;
+	iface->saving_push = anjuta_app_saving_push;
+	iface->saving_pop = anjuta_app_saving_pop;
 }
 
 ANJUTA_TYPE_BEGIN(AnjutaApp, anjuta_app, GTK_TYPE_WINDOW);
